(function() {// from other mail apk

  var CLASS_DY_SCALE = "DY-SCALED";
  var CLASS_DY_CH_HEIGHT = "DY-CH-HEIGHT";
  var CLASS_DY_CH_TOP = "DY-CH-TOP";
  var CLASS_DY_CH_WIDTH = "DY-CH-WIDTH";
  var firstLoad = true;
  var originW = "originW";
  var startTime;
  window.maxWait = 3000;
  var waitTime = maxWait;
  window.historyStack = [];
  window.allowRescale = true;

  function queryAll(doc, selector) {
    return (doc || document.documentElement).querySelectorAll(selector);
  }

  function query(doc, selector) {
    return (doc || document.documentElement).querySelector(selector);
  }

  function attr(el, name, value) {
    if (!!value) {
      el.setAttribute(name, value);
    } else {
      return el.getAttribute(name);
    }
  }

  function getStyleRuleValue(style, selector, sheet) {
    var sheets = typeof sheet !== 'undefined' ? [sheet] : document.styleSheets;
    for (var i = 0, l = sheets.length; i < l; i++) {
      var sheet = sheets[i];
      if (!sheet.cssRules) {
        continue;
      }
      for (var j = 0, k = sheet.cssRules.length; j < k; j++) {
        var rule = sheet.cssRules[j];
        if (rule.selectorText && rule.selectorText.split(',').indexOf(selector) !== -1) {
          return rule.style[style];
        }
      }
    }
    return null;
  }

  function getElementOriWidthOrHeight(ele, name) {
    var w = ele.style[name] || attr(ele, name);
    if (w && w.indexOf("%") == -1 && (!isNaN(w) || w.endsWith("px"))) {
      return parseInt(w);
    }
    var cls = ele.classList;
    for (var i = 0; i < cls.length; ++i) {
      var w = getStyleRuleValue(name, "." + cls[i]);
      if (w && w.indexOf("%") == -1 && (!isNaN(w) || w.endsWith("px"))) {
        return parseInt(w);
      }
    }
    return null;
  }

  function addImportantStyle(ele, name, value) {
    ele.style.setProperty(name, value, "important");
  }

  function cleanStyle(ele, name) {
    ele.style.setProperty(name, "");
  }

  function addClass(ele, clsName) {
    var classes = ele.classList;
    if (!classes.contains(clsName)) {
      classes.add(clsName);
    }
  }

  function removeClass(ele, clsName) {
    var classes = ele.classList;
    if (classes.contains(clsName)) {
      classes.remove(clsName);
    }
  }

  function callNative(n, p) {
    prompt(JSON.stringify({
      name: n,
      text: p
    }));
  }

  function needScale(ele) {
    return ele.id != 'ntes-mail-android-quote-controller';
  }

  function shouldScaleSelf(ele) {
    var tagName = ele.tagName.toLowerCase();
    return ("table" == tagName) || hasBkImage(ele);
  }

  function hasBkImage(ele) {
    var style = getComputedStyle(ele);
    var bkimg = style.backgroundImage;
    var bkr = style.backgroundRepeatX;
    return ("none" != bkimg) && (bkr != "repeat-x");
  }

  function canSetHeight(ele) {
    var tags = ["font", "blockquote", "img"];
    var tagName = ele.tagName.toLowerCase();
    return tags.indexOf(tagName) == -1;
  }

  function shouldRefuseScale(ele) {
    return ele.id == 'ntes-mail-android-quote-controller' ||
      ele.id == 'NE-REFERENCE-PART' ||
      ele.classList.contains('J-reply') ||
      ele.classList.contains('J-cc');
  }

  function tryRemoveWidth(ele, dw, hasFloat) {
    if (hasFloat) {
      return false;
    }
    if (!shouldScaleSelf(ele) && (query(ele, ["[style*='float']"]) == null)) {
      var w = getElementOriWidthOrHeight(ele, "width") || 0;
      attr(ele, originW, w);
      addClass(ele, CLASS_DY_CH_WIDTH);
      addImportantStyle(ele, "width", dw + "px");
      addImportantStyle(ele, "height", "auto");
      return true;
    }
    return false;
  }

  function scaleElement(ele, docWidth, pbw, pwFixed, pzindex, cInfo) {
    if (shouldRefuseScale(ele)) {
      return 0;
    }
    var w = ele.scrollWidth;
    var doScale = shouldScaleSelf(ele);
    var cc = ele.children;
    var cch = 0;
    var pw = getElementOriWidthOrHeight(ele, "width");
    var zIndex = ele.style.zIndex;
    zIndex = "" == zIndex ? pzindex : zIndex;
    var curWFixed = pwFixed || (null != pw);
    if (cInfo) {
      cInfo.hasFloat = getComputedStyle(ele).float != "none";
    }
    if (doScale) {
      lh = scaleNode(ele, docWidth, cch, pbw, pwFixed, null != pw, false);
    } else {
      var info = {
        hasFloat: false
      };
      for (var i = 0; i < cc.length; ++i) {
        cch += scaleElement(cc[i], docWidth, w, curWFixed, zIndex, info);
      }
      lh = scaleNode(ele, docWidth, cch, pbw, pwFixed, null != pw, info.hasFloat);
    }
    if (zIndex != pzindex) {
      return 0;
    } else {
      return lh;
    }
  }

  function scaleNode(ele, docWidth, cch, pbw, pwFixed, curWFixed, hasFloat) {

    if (typeof startTime === "undefined") {
      startTime = new Date();
    }

    if (!window.allowRescale) {
      return false;
    }

    var now = new Date();
    waitTime = waitTime - (now - startTime);
    startTime = now;
    historyStack.push(waitTime);

    var h = ele.offsetHeight;
    var w = ele.scrollWidth
    if (needScale(ele) && w > docWidth && (pbw < w || !pwFixed)) {
      var modify = tryRemoveWidth(ele, docWidth, hasFloat);
      if (modify) {
        var h2 = ele.offsetHeight;
        if (h2 > h) {
          cch += (h2 - h);
        }
      }
      // the scroll width after scaled maybe not right.
      if (modify && (cch != 0) && (ele.scrollWidth == w)) {
        w = ele.offsetWidth;
      } else {
        w = ele.scrollWidth;
      }
      if (canSetHeight(ele)) {
        addClass(ele, CLASS_DY_CH_HEIGHT);
        cch = 0;
      }
      if (w > docWidth) {
        if (!modify) {
          var ow = getElementOriWidthOrHeight(ele, "width") || 0;
          attr(ele, originW, ow);
          addClass(ele, CLASS_DY_CH_WIDTH);
        }
        var s = docWidth / w;
        var oldph = ele.parentElement.offsetHeight;
        addImportantStyle(ele, "-webkit-transform", "scale(" + s + ")");
        addImportantStyle(ele, "-webkit-transform-origin", "0px 0px");
        addImportantStyle(ele, "width", w + "px");
        h = ele.offsetHeight;
        var mr = (Math.round(w * (s - 1)) - 1),
          mb = (Math.round(h * (s - 1)) - 1);
        addImportantStyle(ele, "margin-right", mr + "px");
        addImportantStyle(ele, "margin-bottom", mb + "px");
        addClass(ele, CLASS_DY_SCALE);
        var newph = ele.parentElement.offsetHeight;
        return (oldph - newph + mb + cch + 1);
      }
    } else {
      if (canSetHeight(ele)) {
        addClass(ele, CLASS_DY_CH_HEIGHT);
        cch = 0;
      }
    }
    return cch;
  }

  function shouldScale() {
    var needScale = attr(document.documentElement, "need_scale");
    if ("false" == needScale) {
      return false;
    }
    // 特殊邮件不进行缩放
    return !isSpecialMail();
  }

  function isSpecialMail() {
    if (undefined == window.specialMail) {
      var ele = document.getElementById("mail_master_mail_content_ext");
      window.specialMail = !!ele || isServiceMail();
    }
    return window.specialMail;
  }

  function isServiceMail() {
    console.log(attr(document.documentElement, "mail_type"));
    return attr(document.documentElement, "mail_type") == "service_mail";
  }

  function startScale() {
    if (!shouldScale()) {
      return;
    }
    var docWidth = document.body.offsetWidth;
    if (docWidth <= 0) {
      return;
    }
    var msgDiv = query(undefined, "#dy-content-div");
    msgDiv.style.overflowY = "";
    docWidth = msgDiv.offsetWidth;
    var orgScrollW = msgDiv.scrollWidth;
    var cch = scaleElement(msgDiv.children[0], docWidth, docWidth, false, msgDiv.style.zIndex);
    var h = msgDiv.scrollHeight;
    if (cch < 0) {
      addClass(msgDiv, CLASS_DY_CH_HEIGHT);
      cch = 0;
    }
    if (null == query(msgDiv, ["[style*='absolute']"])) {
      msgDiv.style.overflowY = "hidden";
    }
    return msgDiv.scrollWidth >= orgScrollW;
  }

  window.setAllowRescale = function(isAllow) {
    window.allowRescale = isAllow;
  }

  window.setMaxWait = function(maxWait) {
    window.maxWait = maxWait;
  }

  window.onload = function() {
    processBigAttachment();
  }

  var contentWidth = 0;
  var waitBodyTimer = 0;
  var curKKTitleHeight = 0;
  var firstScale = true;

  function tryToResacle() {
    var times = 0;
    waitBodyTimer = setInterval(function() {
      var bodyW = document.documentElement.clientWidth;
      if ((bodyW == contentWidth) || (++times >= 5)) {
        reScale();
        clearInterval(waitBodyTimer);
      }
    }, 100)
  }

  function adjustAbsoluteTop(nodes, d) {
    for (var i = 0; i < nodes.length; ++i) {
      var node = nodes[i];
      adjustAbsoluteTop(nodes[i].children, d);
      if (node.style.position == "absolute" && node.offsetParent == document.body) {
        var t = parseInt(node.style.top) || d;
        if (!firstLoad || t < d) {
          addClass(node, CLASS_DY_CH_TOP);
          node.style.top = d + "px";
        }
      }
    }
  }

  function cleanAllStyle() {
    var nodes = queryAll(document.documentElement, "." + CLASS_DY_SCALE);
    for (var i = 0; i < nodes.length; ++i) {
      var node = nodes[i];
      removeClass(node, CLASS_DY_SCALE);
      cleanStyle(node, "-webkit-transform");
      cleanStyle(node, "-webkit-transform-origin");
      cleanStyle(node, "margin-right");
      cleanStyle(node, "margin-bottom");
      cleanStyle(node, "height");
      cleanStyle(node, "width");
    }
    nodes = queryAll(document.body, "." + CLASS_DY_CH_HEIGHT);
    for (var i = 0; i < nodes.length; ++i) {
      var node = nodes[i];
      removeClass(node, CLASS_DY_CH_HEIGHT);
      cleanStyle(node, "height");
    }
    nodes = queryAll(document.body, "." + CLASS_DY_CH_WIDTH);
    for (var i = 0; i < nodes.length; ++i) {
      var node = nodes[i];
      removeClass(node, CLASS_DY_CH_WIDTH);
      var w = attr(node, originW);
      if ((parseInt(w) || 0) > 0) {
        addImportantStyle(node, "width", w + "px");
      }
    }
  }

  function processBigAttachment() {
    var div = document.getElementById("QQMailBigAttach");
    if (!!div) {
      div.style.display = "none";
    }
    div = document.getElementById("divNeteaseBigAttach");
    if (!!div) {
      div.style.display = "none";
    }
  }

  window.getDOMInfo = function(option) {
    getDOMInfo(option);
  }

  function getDOMInfo(option) {
    var baseNode = document.querySelector("#dy-content-div");
    var allNodes = baseNode.querySelectorAll("*");
    var finalNodeArr = [];
    var result = {
      hiddenNode: 0,
      blankNode: 0,
      finalNode: 0,
      deep: 0,
      sum: 0,
      nodeType: {},
      weight: 0
    };
    var nodeTypeArr = [];
    var finalNodeDeep = [];

    nodeTypeArr = option.nodeTypeArr.map(function(t) {
      var type = t.toUpperCase();
      result.nodeType[type] = 0;
      return type;
    });

    Array.prototype.forEach.call(allNodes, function(t, idx) {
      if (getComputedStyle(t).display === "none") {
        result.hiddenNode++;
        return true;
      }

      result.sum++;
      if (t.childNodes.length === 0) {
        result.blankNode++;
      }
      if (t.children.length === 0) {
        result.finalNode++;
        finalNodeArr.push(t);
      } else {
        var isAllChildHidden = Array.prototype.every.call(t.children, function(t, idx) {
          return (getComputedStyle(t).display === "none");
        });

        if (isAllChildHidden) {
          result.finalNode++;
          finalNodeArr.push(t);
        }
      }
      if (nodeTypeArr.indexOf(t.nodeName) !== -1) {
        result.nodeType[t.nodeName]++;
      }
    });

    finalNodeArr.forEach(function(t) {
      var deep = 0;
      while (t !== baseNode) {
        deep++;
        t = t.parentElement;
      }
      finalNodeDeep.push(deep);
      result.weight += (deep + 1) * (deep / 2);
    });

    result.deep = Math.max.apply(null, finalNodeDeep);
  }

  window.hiddenNode = function(index) {
    var baseNodeSelector = "#dy-content-div";
    var baseNode = document.querySelector(baseNodeSelector);
    var allNodes = baseNode.querySelectorAll("*");
    var flagNode = null;
    var flagNodeIdx = null;

    Array.prototype.forEach.call(allNodes, function(t, idx) {
      if (idx > index) {
        t.style.display = "none";
      } else if (idx == index) {
        flagNode = t;
      }
      t.style.height = "auto";
    });

    Array.prototype.forEach.call(flagNode.parentElement.childNodes, function(t, idx) {
      if (flagNode == t) {
        flagNodeIdx = idx;
      } else if (flagNodeIdx && (idx > flagNodeIdx)) {
        flagNode.parentElement.removeChild(t);
      }
    });
  }

  window.setNodeLocation = function(index) {
    var baseNodeSelector = "#dy-content-div";
    var baseNode = document.querySelector(baseNodeSelector);
    var curNode = baseNode.querySelectorAll("*")[index];

    var temp = document.createElement("span");
    curNode.parentElement.insertBefore(temp, curNode)

    var scrollTop = temp.getBoundingClientRect().top + window.pageYOffset;

    document.body.scrollTop = scrollTop;
    curNode.parentElement.removeChild(temp)
  }

  window.reScale = function() {
    if (!!window.appHost && !!window.appHost.dontRescale) {
      return;
    }

    if (!window.allowRescale) {
      return;
    }
    startTime = new Date();
    waitTime = maxWait;

    cleanAllStyle();
    var retry = startScale();
    // 一些变态的邮件，第一次缩放后，scroll width不会变小，导致页面
    // 显示过小
    if (retry) {
      cleanAllStyle();
      startScale();
    }
    if (firstScale) {
      firstScale = false;
    }
  }

  window.setBodyPadding = function(top, bottom) {
    addImportantStyle(document.body, "padding-bottom", bottom + "px");
  }

  window.orientationChanged = function(newContentWidth, newScreenWidth) {
    contentWidth = newContentWidth;
    clearInterval(waitBodyTimer);
    tryToResacle();
  }

  window.getText = function() {
    var bodyText = document.body.innerText;
    if (undefined === bodyText) {
      bodyText = "";
    }
  }

  window.getHtml = function(accountId, getHtmlTaskId) {
    var allHtml = document.documentElement.innerHTML;
    if (undefined === allHtml) {
      allHtml = "";
    }
  }
})()